//
// User account management.
//
var userManager = {
	defaultUserKey: 'delicious.users.defaultUser',
	usersKey: 'delicious.users',
	passwordMgmtHost: 'delicious.mozdev.org',

    //
    // Return an array of strings naming all users.
    //
    getUsers: function()
    {
        var users = nsPreferences.copyUnicharPref(this.usersKey, "");
        if (users.length == 0) {
            return new Array;
        } else {
            return users.split(" ");
        }
    },

    //
    // Return a string naming the default user.
    //
    getDefaultUser: function()
    {
        return nsPreferences.copyUnicharPref(this.defaultUserKey, "");
    },

    //
    // Set the default user.
    // If arguments[1] is present, it is considered to be a callback function for asynchronous login.
    //
    setDefaultUser: function(username)
    {
        nsPreferences.setUnicharPref(this.defaultUserKey, username);
        if (this.setDefaultUser.arguments.length == 2) {
            deliciousService.login(this.setDefaultUser.arguments[1]);
        } else {
            deliciousService.login(null);
        }
    },

    //
    // Get the account name for a given user.
    //
    getAccountNameForUser: function(username)
    {
        return nsPreferences.copyUnicharPref(this.usersKey + '.' + username + ".accountname", "");
    },

    //
    // Set the account name for a given user.
    //
    setAccountNameForUser: function(username, accountname)
    {
        nsPreferences.setUnicharPref(this.usersKey + '.' + username + ".accountname", accountname);
    },

    //
    // Get the password for a given account.
    //
    getPasswordForUser: function(username)
    {
        var passwordManager = Components.classes["@mozilla.org/passwordmanager;1"].getService(Components.interfaces.nsIPasswordManager);
        var enumerator = passwordManager.enumerator;
        while (enumerator.hasMoreElements()) {
            var pw = enumerator.getNext().QueryInterface(Components.interfaces.nsIPassword);
            if (pw.host == this.passwordMgmtHost && pw.user == username) {
                return pw.password;
            }
        }
        return "";
    },

    //
    // Set the password for a given user.
    //
    setPasswordForUser: function(username, password)
    {
        var passwordManager = Components.classes["@mozilla.org/passwordmanager;1"].getService(Components.interfaces.nsIPasswordManager);
        passwordManager.addUser(this.passwordMgmtHost, username, password);
    },

    //
    // Create a user. Return a string naming the created account.
    //
    createUser: function(username, password, accountname)
    {
        var users = nsPreferences.copyUnicharPref(this.usersKey, "");
        newUsers = (users + (users.length > 0 ? ' ' : '') + username).split(" ").sort().join(" ");
        nsPreferences.setUnicharPref(this.usersKey, newUsers);
        nsPreferences.setUnicharPref(this.usersKey + '.' + username + ".accountname", accountname);

        var passwordManager = Components.classes["@mozilla.org/passwordmanager;1"].getService(Components.interfaces.nsIPasswordManager);
        passwordManager.addUser(this.passwordMgmtHost, username, password);

        if (users.length == 0) {
            this.setDefaultUser(username);
        }

        return username;
    },

    //
    // Delete a user.
    //
    deleteUser: function(username)
    {
        var users = nsPreferences.copyUnicharPref(this.usersKey, "").split(' ');
        for (i in users) {
            if (username == users[i]) {
                users.splice(i, 1);
                break;
            }
        }
        users = users.join(" ");
        nsPreferences.setUnicharPref(this.usersKey, users);
        
        if (this.getDefaultUser() == username) {
            this.setDefaultUser(users.split(" ")[0]);
        }

        nsPreferences.mPrefService.deleteBranch(this.usersKey + '.' + username);

        var passwordManager = Components.classes["@mozilla.org/passwordmanager;1"].getService(Components.interfaces.nsIPasswordManager);
        passwordManager.removeUser(this.passwordMgmtHost, username);
    },

    //
    // Initialize account manager
    //
    initialize: function()
    {
        // If accounts haven't been used but there's old (not account managed) delicious
        // user info available, create an account by importing the info and set it as the default account.
        //TODO This code can be removed once "most people" are running account managed version.
        const usernameKey = 'delicious.username';
        const passwordKey = 'delicious.password';
        if (this.getDefaultUser().length == 0) {
            var username = nsPreferences.copyUnicharPref(usernameKey, "");
            var password = nsPreferences.copyUnicharPref(passwordKey, "");
            var user = this.createUser(username, password, username);
            this.setDefaultUser(user);

            // Delete the old user preferences.
            nsPreferences.mPrefService.deleteBranch(usernameKey);
            nsPreferences.mPrefService.deleteBranch(passwordKey);
        }
    }
};
